@@ -1,7 +1,7 @@
 /******************************************************************************
     connection.cc from libdap
 
-    Copyright (C) 1998-1999 Patrick Caulfield       patrick@tykepenguin.cix.co.uk
+    Copyright (C) 1998-2008 Christine Caulfield       christine.caulfield@googlemail.com
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -90,6 +90,7 @@ void dap_connection::initialise(int verbosity)
     lasterror   = errstring;
     errstring[0]= '\0';
     closed      = false;
+    connect_timeout = 60;
 
 #ifdef NO_BLOCKING
     blocking_allowed = false; // More useful for debugging
@@ -278,6 +279,10 @@ bool dap_connection::do_connect(const char *node, const char *user,
 	return false;
     }
 
+    // Set connect timeout
+    struct timeval timeout = {connect_timeout, 0};
+    setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, &timeout, sizeof(timeout));
+
     if (::connect(sockfd, (struct sockaddr *)&s,
 		  sizeof(sockaddr)) < 0)
     {
@@ -545,6 +550,12 @@ int dap_connection::get_blocksize()
     return blocksize;
 }
 
+// You (obviously) must call this before connecting
+void dap_connection::set_connect_timeout(int seconds)
+{
+    connect_timeout = seconds;
+}
+
 //
 // Wait for an incoming connection
 // Returns a constructed new dap_connection object
@@ -897,12 +908,12 @@ int dap_connection::send_crc(unsigned short crc)
 // A couple of general utility methods:
 void dap_connection::makelower(char *s)
 {
-    for (unsigned int i=0; i<strlen(s); i++) s[i] = tolower(s[i]);
+    for (unsigned int i=0; s[i]; i++) s[i] = tolower(s[i]);
 }
 
 void dap_connection::makeupper(char *s)
 {
-    for (unsigned int i=0; i<strlen(s); i++) s[i] = toupper(s[i]);
+    for (unsigned int i=0; s[i]; i++) s[i] = toupper(s[i]);
 }
 
 // Always returns false. Sets the error string to strerror(errno)
